function hexStr = fixp2hex(value,fxd)
%FIXP2HEX	Convert a signed value to a hex string represented by fxd-bits
%	hexStr = FIXP2HEX(value,fxd) checks whether 'value' fits in the
%	given 'fxd' bits and if so, returns its hexadecimal representation.
%	'fxd' is supposed to be a two-element vector [N M], where N defines
%	the total number of available bits and M defines the number of bits
%	to the right of the binary point. 
%	If M is too low to exactly represent 'value', 'value' will be truncated,
%	while an error message is issued when N-M should be too low to represent 
%	the	signed integer part of 'value'.
%	'value' can be a vector of fractional decimals.
%
%	See also HEX2FIXP, TOSFIXP, TOUFIXP.  

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004


if ( nargin ~= 2 )
	error( 'syntax: FIXP2HEX(value,fxd) ...' );
end
N = fxd(1);
M = fxd(2);
if ( N <= M )
	error( 'Incorrect values for parameter Fixed ...' );
end

maxPos  =  2^(N-1) -1;
maxNeg  = -2^(N-1);
fracFac =  2^M;
nDigits = ceil(N/4);

value = value(:) * fracFac;
nValues = length(value);
spaceChar = ' ';
hexStr = spaceChar( ones(nValues,nDigits) );
for i = 1:nValues
	testValue = value(i);
	if ( floor(testValue) ~= testValue )
		warning( 'FIXP2HEX: value "%g" doesn''t fit in %d fractional bits ...', ...
																  value(i)/fracFac, M );
		testValue = floor(testValue);
		warning( 'FIXP2HEX: value #%d truncated to %25.20f ...', i, testValue/fracFac );
	end 
	if ((testValue > maxPos) || (testValue < maxNeg))
		error( 'FIXP2HEX: signed value "%g" doesn''t fit in %d integer bits ...', ...
															    value(i)/fracFac, N-M );
	end

	if ( testValue >= 0 )
		hexStr(i,:) = dec2hex( testValue, nDigits );
	else
		hexStr(i,:) = dec2hex( 2^N + testValue, nDigits  );
	end
end
