function view_cir_io( cirFilename, outType, topBottom )
%VIEW_CIR_IO	Graphical view of cir-file description with In- and Outputs shown
%	VIEW_CIR_IO(cirFilename) shows the sequencing graph that is extracted 
%	from the circuit-file 'cirFilename' in a platform specific viewer.
%	An intermediate file 'graph_io.png' will be written in the current 
%	directory.
%
%	VIEW_CIR_IO(cirFilename,outType), where 'outType' is a string, can be used
%	to specify different formats of the 'graph_io'-file.
%	Valid formats besides 'png' are: 'hpg', 'jpg', ...
%	If 'outType' is empty, the 'png' format will be used.
%	
%	VIEW_CIR_IO(cirFilename,outType,topBottom) can be used to specify whether
%	the in and output connections may be displayed inside the graph 
%	(topBottom = 0, the default value), or should be put on seperate top
%	and bottom lines outside the graph (topBottom = 1).
%
%	See also  SHOWGRAPH.

% (c) H.J. Lincklaen Arriens, 
%     Delft University of Technology, November 2004
%									  June 2005, << and >> added


if ~exist( 'outType','var' )
	outType = 'png';
end
if ~exist( 'topBottom','var' )
	topBottom = 0;
end

% read input data
CIRC_Data = parse( cirFilename );
 iconnTbl = CIRC_Data.iconnTbl;
 xconnTbl = CIRC_Data.xconnTbl;
  opTypes = CIRC_Data.opTypes;
  opNames = CIRC_Data.opNames;
 inpNames = CIRC_Data.inpNames;
 outNames = CIRC_Data.outNames;

[nNodes,dummy,shiftInfo] = size(iconnTbl);
adjMat = toAdjMat(iconnTbl);
% compute precedence
[r,c]  = find( adjMat == 1 );
[r,ix] = sort(r);
v = [ r c(ix) ];
if ( shiftInfo ~= 2 )
	iconnTbl(:,:,2) = zeros(nNodes,dummy);
end


%====================================================================================
% Write graph_io.dot 
%====================================================================================

fid = fopen('graph_io.dot','w');

fprintf( fid, '/* %s, generated by ''view_cir_io.m'' */\n', datestr(clock) );
fprintf( fid, '/* %s */\n\n', cirFilename );

fprintf( fid, 'digraph G {\n' );	% 'G' needed buy 'dotty'
%fprintf( fid, 'digraph G {\n\tranksep = 0.75;\n' );
	
tmpStr = 'shape=circle,height=0.6,fixedsize=true,fontname=Courier,fontsize=10';
fprintf( fid, '\n\t{\n\t\tnode [%s]\n', tmpStr );
for i = 1:nNodes
	fprintf( fid, '\t\tn%d [label="%s\\n%s"];\n', i, char(opNames(i)), opTypes(i,:) );
end
fprintf( fid, '\t}\n\n' );


for i = 1:size(v,1)
	fromNode = v(i,1);
	toNode   = v(i,2);
	fromStr  = sprintf( 'n%d', fromNode );
	toStr    = sprintf( 'n%d', toNode   );
	inNo = find( iconnTbl(toNode,2:3,1) == fromNode );
	if ( iconnTbl(toNode,inNo+1,2) == 0 )	% no shift
		fprintf( fid, '\t%s -> %s;\n', fromStr, toStr );
	else
		shiftStr = lfGetShiftStr(iconnTbl(toNode,inNo+1,2));
		fprintf( fid, '\t%s -> %s [style=bold,label="%s"];\n', fromStr, toStr, shiftStr );
	end		
end
fprintf( fid, '\n' );
	

% inputs
inStr   = 'shape=invtrapezium,height=0.4,fixedsize=true,style=bold';
fontStr = 'fontname=Courier,fontsize=10';
fprintf( fid, '\t{\n\t\tnode [%s];\n', [ inStr ',' fontStr ] );
for i = 1:length(inpNames)
	fprintf( fid, '\t\ti%d [label="%s"];\n', i, char(inpNames(i)) );
end
fprintf( fid, '\t}\n\n' );
if topBottom 
	fprintf( fid, '\t{ rank= same;' );
	for i = 1:length(inpNames)
		fprintf( fid, ' "i%d";', i );
	end
	fprintf( fid, ' }\n\n' );
end
for i = 1:length(inpNames)
	[opNo,inNo] = find( iconnTbl(:,2:3,1) == -i );
	if ~isempty(opNo)
		for j = 1:length(opNo)
			fprintf(  fid, '\ti%d -> n%d', i, opNo(j) );
			if ( iconnTbl(opNo(j),inNo(j)+1,2) == 0 )
				fprintf(  fid, ';\n' );
			else
				shiftStr = lfGetShiftStr(iconnTbl(opNo(j),inNo(j)+1,2));
				fprintf(  fid, ' [style=bold,label="%s"];\n', shiftStr );
			end
		end
%	else
%	 	error( 'THIS HAS BEEN CHECKED IN "PARSE": PROBABLY A STRAIGHT-THROUGH CONNECTION' );
	end
end
fprintf( fid, '\n' );

% outputs
nOutputs = size(outNames,1);
outStr  = 'shape=trapezium,height=0.4,fixedsize=true,style=bold';
fprintf( fid, '\t{\n\t\tnode [%s];\n', [ outStr ',' fontStr ] );
for i = 1:nOutputs
	fprintf( fid, '\t\to%d [label="%s"];\n', i, char(outNames(i)) );
end
fprintf( fid, '\t}\n\n' );
if topBottom 
	fprintf( fid, '\t{ rank= same;' );
	for i = 1:length(outNames)
		fprintf( fid, ' "o%d";', i );
	end
	fprintf( fid, ' }\n\n' );
end
for i = 1:nOutputs
	[opNo,dummy] = find( iconnTbl(:,4,1) == i );
	if ~isempty(opNo)
		fprintf( fid, '\tn%d -> o%d', opNo, i );
		if ( iconnTbl(opNo,1,2) == 0 )
			fprintf(  fid, ';\n' );
		else
			shiftStr = lfGetShiftStr(iconnTbl(opNo,1,2));
			fprintf(  fid, ' [style=bold,label="%s"];\n', shiftStr );
		end
	else
		ox = find( strcmp(xconnTbl(:,1),outNames(i)) );
		ix = find( strcmp(inpNames,xconnTbl{ox,2}) );
		fprintf( fid, '\ti%d -> o%d;\n', ix, i );
	end
end
	
fprintf( fid, '\n}\n' );
fclose( fid );

gioName = [ 'graph_io.' outType ];
fConfig;
if ~( dotDir(end) == filesep )
	dotDir = [ dotDir filesep ];
end
eval( [ '!' dotDir 'dot graph_io.dot -T' outType ' -o' gioName ] );
if ispc
	% start imageviewer and let it return
	platform = system_dependent('getos');
	if strcmp(platform(end-1:end),'XP')
		system( 'rundll32 c:\windows\system32\shimgvw.dll ImageView_Fullscreen &' );
		% then show image file
		system( gioName );
	else
		system( [ 'explorer ' gioName ] );
	end
else
    system( [ 'xv ' gioName ' &' ] );
end



%==========================================================================================
%=========  LOCAL FUNCTIONS  ======  LOCAL FUNCTIONS  ======  LOCAL FUNCTIONS  ============
%==========================================================================================

function shiftStr = lfGetShiftStr(kBits)
  if ( kBits < 0 )
    shiftStr = sprintf( ' <<%d ', -kBits );
  else 
    shiftStr = sprintf( ' >>%d ',  kBits );
  end
